import express from "express";
import proxy from "express-http-proxy";
import zlib from "zlib";
import util from "util";
import iconv from "iconv-lite";
import xml from "xml2js";
const gzip: (b: Buffer) => Promise<Buffer> = util.promisify(zlib.gzip);
const gunzip: (b: Buffer) => Promise<Buffer> = util.promisify(zlib.gunzip);

const xmlParser = new xml.Parser();
const xmlBuilder = new xml.Builder({
	xmldec: {
		version: '1.0',
		encoding: 'utf-16',
		standalone: false
	}
});

const baseLocation: string = process.env.BASE_LOCATION;
const baseLocationAccessProtocols: string = process.env.PROTOCOL || "HTTP";

const app = express();
app.use(express.json());
app.use(express.urlencoded({ extended: true }));

app.use('/', proxy("https://downloads.dell.com,", {
	userResDecorator: async (proxyRes, proxyResData, userReq, userRes) => { 
		if (userReq.url.startsWith("/catalog/catalog.xml.gz")) { 
			const content = iconv.decode(await gunzip(proxyResData), "utf16");
			const xmlObject: any = await xmlParser.parseStringPromise(content);
			xmlObject.Manifest.$.baseLocation = baseLocation;
			xmlObject.Manifest.$.baseLocationAccessProtocols = baseLocationAccessProtocols;
			const replaceContent = xmlBuilder.buildObject(xmlObject);
			return await gzip(iconv.encode(replaceContent, "utf16"));
		}
		return proxyResData;
	}
}));
app.listen(8080);
